"""
Synchronous interface for interacting with plumbing API routes.

Clients can use these, but they are less user friendly than our higher level
APIs. They often act as the "plumbing" layer between higher-level APIs.
"""

__all__ = ["Plumbing"]

from fixpoint_common.types.extraction import (
    CreateJsonSchemaExtractionRequest,
    JsonSchemaExtraction,
    CreateQuestionAnswerRecordExtractionRequest,
    QuestionAnswerRecordExtraction,
)
from fixpoint_common.types.parsing import (
    CreateSimpleCrawlUrlParseRequest,
    SimpleCrawlUrlParseResult,
    SimpleWebpageParseResult,
    CreateSimpleWebpageParseRequest,
)
from .._common.plumbing import (
    create_json_schema_extraction,
    create_question_answer_record_extraction,
    create_simple_crawl_url_parse,
    create_simple_webpage_parse,
)
from ._config import Config


class Plumbing:
    """Synchronous interface for plumbing API routes."""

    _config: Config
    _extractions: "_PlumbingExtractions"
    _parses: "_PlumbingParses"

    def __init__(self, config: Config):
        self._config = config
        self._extractions = _PlumbingExtractions(config)
        self._parses = _PlumbingParses(config)

    @property
    def extractions(self) -> "_PlumbingExtractions":
        """Sync interface to extractions API routes."""
        return self._extractions

    @property
    def parses(self) -> "_PlumbingParses":
        """Sync interface to parses API routes."""
        return self._parses


class _PlumbingExtractions:
    _config: Config

    def __init__(self, config: Config):
        self._config = config

    def create_json_schema_extraction(
        self, req: CreateJsonSchemaExtractionRequest
    ) -> JsonSchemaExtraction:
        """Create a structured JSON extraction from a data source.

        Args:
            req (CreateJsonSchemaExtractionRequest): The request containing details for the
                extraction.

        Returns:
            JsonSchemaExtraction: The created extraction.

        Raises:
            HTTPException: If there's an error in the HTTP request to create the
                extraction.
        """
        return create_json_schema_extraction(
            self._config.http_client,
            self._config.core,
            req,
        )

    def create_question_answer_record_extraction(
        self, req: CreateQuestionAnswerRecordExtractionRequest
    ) -> QuestionAnswerRecordExtraction:
        """Create a question and answer ResearchRecord extraction synchronously.

        Args:
            req (CreateQuestionAnswerRecordExtractionRequest): The request
                containing details for the extraction.

        Returns:
            QuestionAnswerRecordExtraction: The created extraction.

        Raises:
            HTTPException: If there's an error in the HTTP request to create the
                extraction.
        """
        return create_question_answer_record_extraction(
            self._config.http_client,
            self._config.core,
            req,
        )


class _PlumbingParses:
    _config: Config

    def __init__(self, config: Config):
        self._config = config

    def create_simple_webpage_parse(
        self, req: CreateSimpleWebpageParseRequest
    ) -> SimpleWebpageParseResult:
        """Create a simple webpage parse asynchronously.

        A simple webpage parse parses a single webpage URL and extracts
        LLM-ready markdown. The text is not chunked.
        """
        return create_simple_webpage_parse(
            self._config.http_client,
            self._config.core,
            req,
        )

    def create_simple_crawl_url_parse(
        self, req: CreateSimpleCrawlUrlParseRequest
    ) -> SimpleCrawlUrlParseResult:
        """Create a simple crawl url parse asynchronously.

        A simple crawl url parse crawls webpages starting from a given URL and
        extracts LLM-ready markdown from all the pages it can find at that URL.
        The parsed text is not chunked.
        """
        return create_simple_crawl_url_parse(
            self._config.http_client,
            self._config.core,
            req,
        )
